{ Copyright (C) by Artem A. Berman }
{ version 1.01 12-Mar-97           }
{     + DaysBeforeDate property    }

unit CheckDate;

interface

uses
  Classes, SysUtils, RegistryEnhanced, Encryp;

type
  TDateValidator = class(TComponent)
  private
    FEncryptor: TTomEncryption;
    FRegistry: TRegistryEnhanced;
    FStartDate:  string;
    FEncKey, 
    FRegKey,
    FRegSubKey,
    FEncRegKey,
    FEncRegSubKey: string;
    FTmpERKey,
    FTmpERSKey: string;
    FDaysToGo,
    FDaysBeforeDate: Integer;
    FJustFine, 
    FNoWay: TNotifyEvent;    
    procedure SetStartDate(strDate: string);
    function GetStartDate: string;
    procedure SetEncRegKey(EKey: string); 
    procedure SetEncRegSubKey(EKey: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    function CheckDate: Boolean;
    procedure ClearKey;
  published
    property StartDate: string read GetStartDate write SetStartDate;
    property EncryptionKey: string read FEncKey write FEncKey;
    property RegistryKey: string read FRegKey write FRegKey;    
    property RegistrySubKey: string read FRegSubKey write FRegSubKey;    
    property EncryptedRegistryKey: string read FEncRegKey write SetEncRegKey;
    property EncryptedRegistrySubKey: string read FEncRegSubKey write SetEncRegSubKey;
    property AllowedDays: Integer read FDaysToGo write FDaysToGo;
    property DaysBeforeDate: Integer read FDaysBeforeDate;
    property OnJustFine: TNotifyEvent read FJustFine write FJustFine;
    property OnNoWay: TNotifyEvent read FNoWay write FNoWay;
  end;

procedure Register;

implementation

constructor TDateValidator.Create(AOwner: TComponent);
begin
  FEncryptor := TTomEncryption.Create(Self);
  FRegistry := TRegistryEnhanced.Create(Self);
  FEncKey := 'DateValidator';
  FDaysToGo := 20;
  inherited Create(AOwner);
end;

destructor TDateValidator.Destroy;
begin
  FEncryptor.Free;
  FRegistry.Free;
  inherited Destroy;
end;

procedure TDateValidator.SetStartDate(strDate: string);
var
  strWritten, strToWrite: string;  
begin
  FStartDate := strDate;
  with FEncryptor do
  begin
   Input := FStartDate;
   Action := atEncryption;
   Key := FEncKey;
   Execute;
   strToWrite := Output;
  end;
  with FRegistry do
  begin
    if FTmpERKey = '' then Key := FRegKey else Key := FTmpERKey;
    if FTmpERSKey = '' then strWritten := GetValue(FRegSubKey) else
    strWritten := GetValue(FTmpERSKey);
    if strWritten = '' then
    begin
      if FTmpERSKey = '' then SetValue(FRegSubKey, strToWrite) else
                         SetValue(FTmpERSKey, strToWrite);
    end else Exit;
  end;
end;

function TDateValidator.GetStartDate;
var
  strReaded: string;
begin
  with FRegistry do
  begin
    if FTmpERKey = '' then Key := FRegKey else Key := FTmpERKey;
    if FTmpERSKey = '' then strReaded := GetValue(FRegSubKey) else
                            strReaded := GetValue(FTmpERSKey)
  end;
  with FEncryptor do
  begin
   Input := strReaded;
   Action := atDecryption;
   Key := FEncKey;
   Execute;
   Result := Output;
  end;
end;

procedure TDateValidator.ClearKey;
begin
  FRegistry.DeleteKey('');
end;

function TDateValidator.CheckDate;
var
  initDay, initMonth, initYear: Word;
  curDay, curMonth, curYear: Word;
  totalDays: word;
  initDate, curDate: TDateTime;

begin
  Result := False;
  initDate := StrToDate(StartDate);
  curDate := Date;  

  DecodeDate(initDate, initYear, initMonth, initDay);
  DecodeDate(curDate, curYear, curMonth, curDay);

  if initMonth = curMonth then 
  begin
    FDaysBeforeDate := curDay - initDay;
    (if FDaysBeforeDate <= FDaysToGo then Result := True)
  end
  else begin
    case initMonth of
      1 : totalDays := 31;
      2 : totalDays := 28;
      3 : totalDays := 31;
      4 : totalDays := 30;
      5 : totalDays := 31;
      6 : totalDays := 30;
      7 : totalDays := 31;
      8 : totalDays := 31;
      9 : totalDays := 30;
      10: totalDays := 31;
      11: totalDays := 30;
      12: totalDays := 31;
    end;
    FDaysBeforeDate := totalDays - initDay + curDay;
    if FDaysBeforeDate <= FDaysToGo then Result := True;
  end;  
  if Result AND Assigned(FJustFine) then FJustFine(Self)
  else
  if not Result AND Assigned(FNoWay) then FNoWay(Self)
end;

procedure TDateValidator.SetEncRegKey(EKey: string); 
begin
  if EKey <> FEncRegKey then FEncRegKey := EKey;
  with FEncryptor do
  begin
   Input := FEncRegKey;
   Action := atDecryption;
   Key := FEncKey;
   Execute;
   FTmpERKey := Output;
  end;
end;

procedure TDateValidator.SetEncRegSubKey(EKey: string);
begin
  if EKey <> FEncRegSubKey then FEncRegSubKey := EKey;
  with FEncryptor do
  begin
   Input := FEncRegSubKey;
   Action := atDecryption;
   Key := FEncKey;
   Execute;
   FTmpERSKey := Output;  
  end;
end;

procedure Register;
begin
  RegisterComponents('Custom', [TDateValidator]);
end;

end.